/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_SLAMLOCALEGOODOM_H
#define DPC_IMPL_TYPE_SLAMLOCALEGOODOM_H

#include <cfloat>
#include <cmath>
#include "impl_type_double.h"
#include "impl_type_string.h"
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector3f.h"

namespace dpc {
struct SLAMLocalEgoOdom {
    ::Double ego_odom_time_ns{ 0.0 };
    ::String status_inf;
    ::String debug_inf;
    ::UInt8 origin_id{ 0U };
    ::Float roll{ 0.0F };
    ::Float pitch{ 0.0F };
    ::Float yaw{ 0.0F };
    ::dpc::Vector3f position;
    ::dpc::Vector3f position_std_dev;
    ::dpc::Vector3f attitude_std_dev;
    ::dpc::Vector3f linear_velocity;
    ::dpc::Vector3f linear_acceleration;
    ::dpc::Vector3f angular_velocity;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(ego_odom_time_ns);
        fun(status_inf);
        fun(debug_inf);
        fun(origin_id);
        fun(roll);
        fun(pitch);
        fun(yaw);
        fun(position);
        fun(position_std_dev);
        fun(attitude_std_dev);
        fun(linear_velocity);
        fun(linear_acceleration);
        fun(angular_velocity);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(ego_odom_time_ns);
        fun(status_inf);
        fun(debug_inf);
        fun(origin_id);
        fun(roll);
        fun(pitch);
        fun(yaw);
        fun(position);
        fun(position_std_dev);
        fun(attitude_std_dev);
        fun(linear_velocity);
        fun(linear_acceleration);
        fun(angular_velocity);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("ego_odom_time_ns", ego_odom_time_ns);
        fun("status_inf", status_inf);
        fun("debug_inf", debug_inf);
        fun("origin_id", origin_id);
        fun("roll", roll);
        fun("pitch", pitch);
        fun("yaw", yaw);
        fun("position", position);
        fun("position_std_dev", position_std_dev);
        fun("attitude_std_dev", attitude_std_dev);
        fun("linear_velocity", linear_velocity);
        fun("linear_acceleration", linear_acceleration);
        fun("angular_velocity", angular_velocity);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("ego_odom_time_ns", ego_odom_time_ns);
        fun("status_inf", status_inf);
        fun("debug_inf", debug_inf);
        fun("origin_id", origin_id);
        fun("roll", roll);
        fun("pitch", pitch);
        fun("yaw", yaw);
        fun("position", position);
        fun("position_std_dev", position_std_dev);
        fun("attitude_std_dev", attitude_std_dev);
        fun("linear_velocity", linear_velocity);
        fun("linear_acceleration", linear_acceleration);
        fun("angular_velocity", angular_velocity);
    }

    friend bool operator==(const ::dpc::SLAMLocalEgoOdom& lhs, const ::dpc::SLAMLocalEgoOdom& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.ego_odom_time_ns - rhs.ego_odom_time_ns)) < DBL_EPSILON) && (lhs.status_inf == rhs.status_inf) && (lhs.debug_inf == rhs.debug_inf) && (lhs.origin_id == rhs.origin_id) && (fabs(static_cast<double>(lhs.roll - rhs.roll)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.pitch - rhs.pitch)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.yaw - rhs.yaw)) < DBL_EPSILON) && (lhs.position == rhs.position) && (lhs.position_std_dev == rhs.position_std_dev) && (lhs.attitude_std_dev == rhs.attitude_std_dev) && (lhs.linear_velocity == rhs.linear_velocity) && (lhs.linear_acceleration == rhs.linear_acceleration) && (lhs.angular_velocity == rhs.angular_velocity);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_SLAMLOCALEGOODOM_H
